<?php

namespace Drupal\yasm_blocks\Plugin\Block;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\Group;
use Drupal\yasm\Services\GroupsStatisticsInterface;
use Drupal\yasm\Services\YasmBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'YASM group counts' Block.
 *
 * @Block(
 *   id = "yasm_block_group",
 *   admin_label = @Translation("YASM current group counts"),
 *   category = @Translation("YASM"),
 * )
 */
class GroupBlock extends YasmBlock implements ContainerFactoryPluginInterface {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entities statistics service.
   *
   * @var \Drupal\yasm\Services\GroupsStatisticsInterface
   */
  protected $groupsStatistics;

  /**
   * The access manager service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $routeMatch;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function build() {
    if ($this->moduleHandler->moduleExists('group')) {
      if ($group = $this->getGroupFromRoute()) {

        $config = $this->getConfiguration();
        $with_icons = isset($config['with_icons']) ? $config['with_icons'] : TRUE;

        if (isset($config['block_style']) && 'cards' === $config['block_style']) {
          $build = $this->buildBlockColumns($this->getGroupCards($group, $with_icons));
        }
        elseif (isset($config['block_style']) && 'counters' === $config['block_style']) {
          $build = $this->buildBlockColumns($this->getGroupCards($group, $with_icons));
          $build['#attributes']['class'][] = 'yasm-counters';
          $build['#attached']['library'][] = 'yasm_blocks/counters';
        }
        else {
          $build = [
            '#theme' => 'item_list',
            '#list_type' => 'ul',
            '#items' => $this->getGroupCards($group, $with_icons, TRUE),
          ];
        }

        if (!empty($config['with_icons']) && !empty($config['attach_fontawesome'])) {
          $build['#attached']['library'][] = 'yasm/fontawesome';
        }

        return [
          '#theme' => 'yasm_wrapper',
          '#content' => $build,
          '#attributes' => [
            'class' => ['yasm-block', 'yasm-block-group'],
          ],
          '#cache' => [
            'contexts' => ['group:' . $group->id()],
            'max-age'  => 3600,
          ],
        ];
      }
    }

    return [];
  }

  /**
   * Get group counts cards.
   */
  private function getGroupCards($group, $with_icons = TRUE, $list = FALSE) {
    $cards = [];

    // Group contents.
    $label = $this->t('Contents');
    $count = $this->groupsStatistics->countContents($group);
    $picto = $with_icons ? 'far fa-file-alt' : '';

    $cards[] = $this->buildBlockItem($label, $count, $picto, $list);

    // Group members.
    $label = $this->t('Members');
    $count = $this->groupsStatistics->countMembers($group);
    $picto = $with_icons ? 'fas fa-user' : '';

    $cards[] = $this->buildBlockItem($label, $count, $picto, $list);

    return $cards;
  }

  /**
   * Get group from current route.
   */
  private function getGroupFromRoute() {
    if ($group = $this->routeMatch->getParameters()->get('group')) {
      if ($group instanceof Group) {
        return $group;
      }

      if (is_numeric($group)) {
        if ($group = $this->entityTypeManager->getStorage('group')->load($group)) {
          return $group;
        }
      }
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ModuleHandlerInterface $module_handler,
    GroupsStatisticsInterface $groups_statistics,
    CurrentRouteMatch $route_match,
    EntityTypeManagerInterface $entityTypeManager,
    YasmBuilderInterface $yasm_builder
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $yasm_builder);

    $this->moduleHandler     = $module_handler;
    $this->groupsStatistics  = $groups_statistics;
    $this->routeMatch        = $route_match;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('module_handler'),
      $container->get('yasm.groups_statistics'),
      $container->get('current_route_match'),
      $container->get('entity_type.manager'),
      $container->get('yasm.builder')
    );
  }

}
